=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-02-15 06:08:08 +0000
+++ bzrlib/builtins.py	2006-02-16 11:19:47 +0000
@@ -1450,11 +1450,29 @@
     this command. When the default format has changed you may also be warned
     during other operations to upgrade.
     """
+    # NB: this is used from the class without creating an instance, which is
+    # why it does not have a self parameter.
+    def get_format_type(typestring):
+        """Parse and return a format specifier."""
+        if typestring == "metadir":
+            return bzrdir.BzrDirMetaFormat1
+        msg = "No known bzr-dir format %s. Supported types are: metadir\n" %\
+            (typestring)
+        raise BzrCommandError(msg)
+
     takes_args = ['url?']
-
-    def run(self, url='.'):
+    takes_options = [
+                     Option('format', 
+                            help='Upgrade to a specific format rather than the'
+                                 ' current default format. Currently this '
+                                 ' option only accepts =metadir',
+                            type=get_format_type),
+                    ]
+
+
+    def run(self, url='.', format=None):
         from bzrlib.upgrade import upgrade
-        upgrade(url)
+        upgrade(url, format)
 
 
 class cmd_whoami(Command):

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-02-16 09:31:35 +0000
+++ bzrlib/bzrdir.py	2006-02-16 11:19:47 +0000
@@ -362,15 +362,21 @@
         self.transport = _transport.clone('.bzr')
         self.root_transport = _transport
 
-    def needs_format_update(self):
+    def needs_format_update(self, format=None):
         """Return true if this bzrdir needs update_format run on it.
         
         For instance, if the repository format is out of date but the 
         branch and working tree are not, this should return True.
+
+        :param format: Optional parameter indicating a specific desired
+                       format we want to end up at.
         """
         # for now, if the format is not the same as the system default,
-        # an upgrade is needed.
-        return not isinstance(self._format, BzrDirFormat.get_default_format().__class__)
+        # an upgrade is needed. In the future we will want to scan
+        # the individual repository/branch/checkout formats too
+        if format is None:
+            format = BzrDirFormat.get_default_format().__class__
+        return not isinstance(self._format, format)
 
     @staticmethod
     def open_unsupported(base):
@@ -642,7 +648,7 @@
         from bzrlib.repository import RepositoryFormat4
         return RepositoryFormat4().initialize(self, shared)
 
-    def needs_format_update(self):
+    def needs_format_update(self, format=None):
         """Format 4 dirs are always in need of updating."""
         return True
 
@@ -754,8 +760,9 @@
             pass
         return self.transport.clone('checkout')
 
-    def needs_format_update(self):
+    def needs_format_update(self, format=None):
         """See BzrDir.needs_format_update()."""
+        # currently there are no possible updates to meta1 formats.
         return False
 
     def open_branch(self, unsupported=False):
@@ -823,7 +830,7 @@
         """Return the ASCII format string that identifies this format."""
         raise NotImplementedError(self.get_format_string)
 
-    def get_updater(self):
+    def get_updater(self, format=None):
         """Return the updater to use to convert bzrdirs needing updates.
 
         This returns a bzrlib.bzrdir.Converter object.
@@ -831,6 +838,9 @@
         This should return the best upgrader to step this format towards the
         current default format. In the case of plugins we can/shouold provide
         some means for them to extend the range of returnable converters.
+
+        :param format: Optional format to override the default foramt of the 
+                       library.
         """
         raise NotImplementedError(self.get_updater)
 
@@ -926,8 +936,9 @@
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 0.0.4\n"
 
-    def get_updater(self):
+    def get_updater(self, format=None):
         """See BzrDirFormat.get_updater()."""
+        # there is one and only one upgrade path here.
         return ConvertBzrDir4To5()
         
     def initialize(self, url):
@@ -963,8 +974,9 @@
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 5\n"
 
-    def get_updater(self):
+    def get_updater(self, format=None):
         """See BzrDirFormat.get_updater()."""
+        # there is one and only one upgrade path here.
         return ConvertBzrDir5To6()
         
     def initialize(self, url, _cloning=False):
@@ -1001,8 +1013,9 @@
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 6\n"
 
-    def get_updater(self):
+    def get_updater(self, format=None):
         """See BzrDirFormat.get_updater()."""
+        # there is one and only one upgrade path here.
         return ConvertBzrDir6ToMeta()
         
     def initialize(self, url, _cloning=False):
@@ -1471,7 +1484,7 @@
         self.total = 20 # the steps we know about
         self.garbage_inventories = []
 
-        self.pb.note('starting upgrade from format 5 to 6')
+        self.pb.note('starting upgrade from format 6 to metadir')
         self.bzrdir._control_files.put_utf8('branch-format', "Converting to format 6")
         # its faster to move specific files around than to open and use the apis...
         # first off, nuke ancestry.weave, it was never used.

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2006-02-16 08:18:32 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2006-02-16 11:19:47 +0000
@@ -31,11 +31,14 @@
     """
 
     def note(self, fmt_string, *args, **kwargs):
-        """See progress.ProgressBae.note()."""
+        """See progress.ProgressBar.note()."""
         print fmt_string % args
 
     def progress_bar(self):
         return self
+        
+    def update(self, message, count, total):
+        """See progress.ProgressBar.update()."""
 
 
 class TestWithUpgradableBranches(TestCaseWithTransport):
@@ -103,3 +106,26 @@
     def test_ugrade_branch_in_repo(self):
         # upgrading a branch in a repo should warn about not upgrading the repo
         pass
+
+    def test_upgrade_explicit_metaformat(self):
+        # users can force an upgrade to metadir format.
+        url = get_transport(self.get_url('format_5_branch')).base
+        # check --format takes effect
+        bzrdir.BzrDirFormat.set_default_format(bzrdir.BzrDirFormat5())
+        (out, err) = self.run_bzr_captured(
+            ['upgrade', '--format=metadir', url])
+        self.assertEqualDiff("""starting upgrade of %s
+making backup of tree history
+%s.bzr has been backed up to %s.bzr.backup
+if conversion fails, you can move this directory back to .bzr
+if it succeeds, you can remove this directory if you wish
+starting upgrade from format 5 to 6
+adding prefixes to weaves
+adding prefixes to revision-store
+starting upgrade from format 6 to metadir
+finished
+""" % (url, url, url), out)
+        self.assertEqualDiff("", err)
+        self.assertTrue(isinstance(
+            bzrdir.BzrDir.open(self.get_url('format_5_branch'))._format,
+            bzrdir.BzrDirMetaFormat1))

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-02-16 09:31:35 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-02-16 11:19:47 +0000
@@ -984,13 +984,13 @@
             # if its updatable there must be an updater
             self.assertTrue(isinstance(dir._format.get_updater(),
                                        bzrdir.Converter))
-        dir.needs_format_update()
+        dir.needs_format_update(None)
 
     def test_upgrade_new_instance(self):
         """Does an available updater work ?."""
         dir = self.make_bzrdir('.')
         if dir.can_update_format():
-            dir._format.get_updater().convert(dir, ui.ui_factory.progress_bar())
+            dir._format.get_updater(None).convert(dir, ui.ui_factory.progress_bar())
             # and it should pass 'check' now.
             check(bzrdir.BzrDir.open(self.get_url('.')).open_branch(), False)
 

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2006-02-16 08:51:52 +0000
+++ bzrlib/upgrade.py	2006-02-16 11:19:47 +0000
@@ -27,7 +27,8 @@
 
 class Convert(object):
 
-    def __init__(self, url):
+    def __init__(self, url, format):
+        self.format = format
         self.bzrdir = BzrDir.open_unsupported(url)
         if self.bzrdir.root_transport.is_readonly():
             raise errors.UpgradeReadonly
@@ -41,21 +42,15 @@
             self.pb.note("This is a checkout. The branch (%s) needs to be "
                          "upgraded separately.",
                          branch.bzrdir.root_transport.base)
-        if not self.bzrdir.needs_format_update():
+        if not self.bzrdir.needs_format_update(self.format):
             raise errors.UpToDateFormat(self.bzrdir._format)
         if not self.bzrdir.can_update_format():
             raise errors.BzrError("cannot upgrade from branch format %s" %
                            self.bzrdir._format)
         self.pb.note('starting upgrade of %s', self.transport.base)
         self._backup_control_dir()
-        while self.bzrdir.needs_format_update():
-            converter = self.bzrdir._format.get_updater()
-            self.bzrdir = converter.convert(self.bzrdir, self.pb)
-        if isinstance(self.bzrdir._format, BzrDirFormat4):
-            converter = ConvertBzrDir4To5()
-            self.bzrdir = converter.convert(self.bzrdir, self.pb)
-        if isinstance(self.bzrdir._format, BzrDirFormat5):
-            converter = ConvertBzrDir5To6()
+        while self.bzrdir.needs_format_update(self.format):
+            converter = self.bzrdir._format.get_updater(self.format)
             self.bzrdir = converter.convert(self.bzrdir, self.pb)
         self.pb.note("finished")
 
@@ -68,5 +63,6 @@
         self.pb.note('if conversion fails, you can move this directory back to .bzr')
         self.pb.note('if it succeeds, you can remove this directory if you wish')
 
-def upgrade(url):
-    Convert(url)
+def upgrade(url, format=None):
+    """Upgrade to format, or the default bzrdir format if not supplied."""
+    Convert(url, format)

